package com.unis.module.coding.service.fmxx;

import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import com.unis.module.coding.controller.admin.fmxx.vo.*;
import com.unis.module.coding.dal.dataobject.fmxx.FmxxDO;
import com.unis.framework.common.pojo.PageResult;
import com.unis.framework.common.pojo.PageParam;
import com.unis.framework.common.util.object.BeanUtils;

import com.unis.module.coding.dal.mysql.fmxx.FmxxMapper;

import static com.unis.framework.common.exception.util.ServiceExceptionUtil.exception;
import static com.unis.module.core.enums.ErrorCodeConstants.FMXX_NOT_EXISTS;

/**
 * 赋码结果信息 Service 实现类
 *
 * @author 超级管理员
 */
@Service
@Validated
public class FmxxServiceImpl implements FmxxService {

    @Resource
    private FmxxMapper fmxxMapper;

    @Override
    public Long createFmxx(FmxxSaveReqVO createReqVO) {
        // 插入
        FmxxDO fmxx = BeanUtils.toBean(createReqVO, FmxxDO.class);
        fmxxMapper.insert(fmxx);
        // 返回
        return fmxx.getId();
    }

    @Override
    public void updateFmxx(FmxxSaveReqVO updateReqVO) {
        // 校验存在
        validateFmxxExists(updateReqVO.getId());
        // 更新
        FmxxDO updateObj = BeanUtils.toBean(updateReqVO, FmxxDO.class);
        fmxxMapper.updateById(updateObj);
    }

    @Override
    public void deleteFmxx(Long id) {
        // 校验存在
        validateFmxxExists(id);
        // 删除
        fmxxMapper.deleteById(id);
    }

    private void validateFmxxExists(Long id) {
        if (fmxxMapper.selectById(id) == null) {
            throw exception(FMXX_NOT_EXISTS);
        }
    }

    @Override
    public FmxxDO getFmxx(Long id) {
        return fmxxMapper.selectById(id);
    }

    @Override
    public PageResult<FmxxDO> getFmxxPage(FmxxPageReqVO pageReqVO) {
        return fmxxMapper.selectPage(pageReqVO);
    }

}